@@ -24,49 +24,168 @@
 
 #include "config.h"
 
-#include <stdio.h>
+#include <assert.h>
 #include <gnome.h>
 #include <guile/gh.h>
+#include <stdio.h>
 #include <time.h>
-#include <assert.h>
 
-#include "top-level.h"
-#include "messages_i18n.h"
-#include "ui-callbacks.h"
-#include "RegWindow.h"
-#include "window-register.h"
-#include "account-tree.h"
-#include "MultiLedger.h"
 #include "FileDialog.h"
-#include "splitreg.h"
-#include "dialog-find-transactions.h"
-#include "window-help.h"
+#include "MultiLedger.h"
 #include "Query.h"
+#include "RegWindow.h"
+#include "account-tree.h"
+#include "dialog-find-transactions.h"
+#include "dialog-utils.h"
+#include "glade-cb-gnc-dialogs.h"
+#include "glade-gnc-dialogs.h"
+#include "gnc-amount-edit.h"
+#include "gnc-component-manager.h"
 #include "gnc-dateedit.h"
+#include "gnc-engine-util.h"
+#include "gnc-ui.h"
+#include "messages.h"
+#include "splitreg.h"
+#include "window-help.h"
+#include "window-register.h"
 
-/********************************************************************\
- * gnc_ui_find_transactions_dialog_create
- * make a new print check dialog and wait for it.
-\********************************************************************/
+#define DIALOG_FIND_TRANS_CM_CLASS "dialog-find-trans"
+
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_GUI;
+
+struct _SelectDateDialog {
+  GtkWidget * dialog;
+  GtkWidget * cal;
+  GtkWidget * entry_1;
+  GtkWidget * entry_2;
+  GtkWidget * entry_3;
+  char      * ymd_format;
+};
+
+struct _FindTransactionsDialog {
+  GtkWidget  * dialog;
+  Query      * q;
+  Query      * ledger_q;
+
+  char       * ymd_format;
+
+  int        search_type;
+
+  GtkWidget  * new_search_radiobutton;
+  GtkWidget  * narrow_search_radiobutton;
+  GtkWidget  * add_search_radiobutton;
+  GtkWidget  * delete_search_radiobutton;
+
+  GtkWidget  * match_accounts_picker;
+  GtkWidget  * match_accounts_scroller;
+  GtkWidget  * account_tree;
+
+  GtkWidget  * date_start_toggle;
+  GtkWidget  * date_start_frame;
+  GtkWidget  * date_start_entry;
+
+  GtkWidget  * date_end_toggle;
+  GtkWidget  * date_end_frame;
+  GtkWidget  * date_end_entry;
+
+  GtkWidget  * description_entry;
+  GtkWidget  * description_case_toggle;
+  GtkWidget  * description_regexp_toggle;
+
+  GtkWidget  * number_entry;
+  GtkWidget  * number_case_toggle;
+  GtkWidget  * number_regexp_toggle;
+
+  GtkWidget  * credit_debit_picker;
+  GtkWidget  * amount_comp_picker;
+  GtkWidget  * amount_edit;
+
+  GtkWidget  * memo_entry;
+  GtkWidget  * memo_case_toggle;
+  GtkWidget  * memo_regexp_toggle;
+
+  GtkWidget  * shares_comp_picker;
+  GtkWidget  * shares_edit;
+
+  GtkWidget  * price_comp_picker;
+  GtkWidget  * price_edit;
+  
+  GtkWidget  * action_entry;
+  GtkWidget  * action_case_toggle;
+  GtkWidget  * action_regexp_toggle;
+
+  GtkWidget  * cleared_not_cleared_toggle;
+  GtkWidget  * cleared_cleared_toggle;
+  GtkWidget  * cleared_reconciled_toggle;
+
+  GtkWidget  * balance_balanced_toggle;
+  GtkWidget  * balance_not_balanced_toggle;
+
+  GtkWidget  * tag_entry;
+  GtkWidget  * tag_case_toggle;
+  GtkWidget  * tag_regexp_toggle;
+};
+
+
+static int
+gnc_find_dialog_close_cb(GnomeDialog *dialog, gpointer data) {
+  FindTransactionsDialog * ftd = data;
+
+  ftd->dialog = NULL;
+
+  xaccFreeQuery (ftd->q);
+  ftd->q = NULL;
+
+  g_free (ftd->ymd_format);
+  ftd->ymd_format = NULL;
+
+  gnc_unregister_gui_component_by_data (DIALOG_FIND_TRANS_CM_CLASS, ftd);
+
+  g_free (ftd);
+
+  return FALSE;
+}
+
+static void
+close_handler (gpointer user_data)
+{
+  FindTransactionsDialog * ftd = user_data;
+
+  gnome_dialog_close (GNOME_DIALOG (ftd->dialog));
+}
 
 FindTransactionsDialog * 
 gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
-  FindTransactionsDialog * ftd = g_new0(FindTransactionsDialog, 1);  
+  FindTransactionsDialog * ftd = g_new0(FindTransactionsDialog, 1);
+  GtkWidget *box;
+  GtkWidget *edit;
+  GtkWidget *notebook;
+  gint page_num;
 
   /* call the glade-defined creator */
   ftd->dialog = create_Find_Transactions();
-  ftd->ledger = orig_ledg;
 
   if(orig_ledg) {
-    ftd->q = orig_ledg->query;
+    ftd->q = xaccQueryCopy (xaccLedgerDisplayGetQuery (orig_ledg));
+    ftd->ledger_q = xaccLedgerDisplayGetQuery (orig_ledg);
   }
   else {
     ftd->q = NULL;
+    ftd->ledger_q = NULL;
   }
-  
+
   /* initialize the radiobutton state vars */
   ftd->search_type = 0;
 
+  /* remove the tags page (unfinished) */
+  notebook = gtk_object_get_data (GTK_OBJECT (ftd->dialog), "find_notebook");
+  box = gtk_object_get_data (GTK_OBJECT (ftd->dialog), "tags_frame");
+
+  page_num = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), box);
+  if (page_num >= 0)
+    gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
+
   /* find the important widgets */
   ftd->new_search_radiobutton = 
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "new_search_radiobutton");
@@ -120,8 +239,12 @@ gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "credit_debit_picker");
   ftd->amount_comp_picker =
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "amount_comp_picker");
-  ftd->amount_entry =
-    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "amount_entry");
+
+  box = gtk_object_get_data(GTK_OBJECT(ftd->dialog), "amount_box");
+  edit = gnc_amount_edit_new ();
+  gtk_widget_show (edit);
+  gtk_box_pack_start (GTK_BOX (box), edit, TRUE, TRUE, 0);
+  ftd->amount_edit = edit;
 
   ftd->memo_entry =
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "memo_entry");
@@ -132,13 +255,21 @@ gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
 
   ftd->shares_comp_picker =
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "shares_comp_picker");
-  ftd->shares_entry =
-    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "shares_entry");
+
+  box = gtk_object_get_data(GTK_OBJECT(ftd->dialog), "shares_box");
+  edit = gnc_amount_edit_new ();
+  gtk_widget_show (edit);
+  gtk_box_pack_start (GTK_BOX (box), edit, TRUE, TRUE, 0);
+  ftd->shares_edit = edit;
 
   ftd->price_comp_picker =
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "price_comp_picker");
-  ftd->price_entry =
-    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "price_entry");
+
+  box = gtk_object_get_data(GTK_OBJECT(ftd->dialog), "price_box");
+  edit = gnc_amount_edit_new ();
+  gtk_widget_show (edit);
+  gtk_box_pack_start (GTK_BOX (box), edit, TRUE, TRUE, 0);
+  ftd->price_edit = edit;
 
   ftd->action_entry =
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "action_entry");
@@ -154,6 +285,19 @@ gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
   ftd->cleared_not_cleared_toggle = 
     gtk_object_get_data(GTK_OBJECT(ftd->dialog), "cleared_not_cleared_toggle");
 
+  ftd->balance_balanced_toggle = 
+    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "balance_balanced_toggle");
+  ftd->balance_not_balanced_toggle = 
+    gtk_object_get_data(GTK_OBJECT(ftd->dialog),
+                        "balance_not_balanced_toggle");
+
+  ftd->tag_entry =
+    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "tag_entry");
+  ftd->tag_case_toggle =
+    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "tag_case_toggle");
+  ftd->tag_regexp_toggle =
+    gtk_object_get_data(GTK_OBJECT(ftd->dialog), "tag_regexp_toggle");
+
   /* add an account picker to the first tab */
   ftd->account_tree = gnc_account_tree_new();
   gtk_clist_column_titles_hide(GTK_CLIST(ftd->account_tree));
@@ -178,7 +322,6 @@ gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
   /* set data so we can find the struct in callbacks */
   gtk_object_set_data(GTK_OBJECT(ftd->dialog), "find_transactions_structure",
                       ftd);
-  
 
   /* if there's no original query, make the narrow, add, delete 
    * buttons inaccessible */
@@ -198,8 +341,15 @@ gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
                                  (ftd->narrow_search_radiobutton),
                                  1);
   }
-  
+
+  gnc_register_gui_component (DIALOG_FIND_TRANS_CM_CLASS,
+                              NULL, close_handler, ftd);
+
+  gtk_signal_connect(GTK_OBJECT(ftd->dialog), "close",
+                     GTK_SIGNAL_FUNC(gnc_find_dialog_close_cb), ftd);
+
   gtk_widget_show_all(ftd->dialog);
+
   return ftd;
 }
 
@@ -210,13 +360,12 @@ gnc_ui_find_transactions_dialog_create(xaccLedgerDisplay * orig_ledg) {
 
 void
 gnc_ui_find_transactions_dialog_destroy(FindTransactionsDialog * ftd) {
-  gnome_dialog_close(GNOME_DIALOG(ftd->dialog));
-  
-  ftd->dialog = NULL;
-  g_free(ftd->ymd_format);
-  g_free(ftd);
+  if (!ftd)
+    return;
+
+  gnc_close_gui_component_by_data (DIALOG_FIND_TRANS_CM_CLASS, ftd);
 }
-  
+
 
 /********************************************************************\
  * callbacks for radio button selections 
@@ -245,7 +394,7 @@ gnc_ui_find_transactions_dialog_search_type_cb(GtkToggleButton * tb,
 
 void
 gnc_ui_find_transactions_dialog_cancel_cb(GtkButton * button, 
-                                    gpointer user_data) {
+                                          gpointer user_data) {
   FindTransactionsDialog * ftd =
     gtk_object_get_data(GTK_OBJECT(user_data), "find_transactions_structure");
 
@@ -260,7 +409,7 @@ gnc_ui_find_transactions_dialog_cancel_cb(GtkButton * button,
 void
 gnc_ui_find_transactions_dialog_help_cb(GtkButton * button, 
                                         gpointer user_data) {
-  helpWindow(NULL, HELP_STR, HH_FIND_TRANSACTIONS);
+  helpWindow(NULL, NULL, HH_FIND_TRANSACTIONS);
 }
 
 
@@ -312,27 +461,29 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
   GtkWidget              * dialog = user_data;
   FindTransactionsDialog * ftd = 
     gtk_object_get_data(GTK_OBJECT(dialog), "find_transactions_structure");
-  
+
+  xaccLedgerDisplay *ledger;
+
   GList   * selected_accounts;
-  GList   * cur;
   char    * descript_match_text;
   char    * memo_match_text;
   char    * number_match_text;
   char    * action_match_text;
+  char    * tag_match_text;
 
   int     search_type = ftd->search_type;
 
   float   amt_temp;
   int     amt_type;
   Query   * q, * q2;
-  Account ** acclist;
-  int     i;
+  Query   * new_q;
   gboolean new_ledger = FALSE;
 
   int    use_start_date, use_end_date;
   time_t start_date, end_date;
 
   int c_cleared, c_notcleared, c_reconciled;
+  int b_balanced, b_not_balanced;
 
   if(search_type == 0) {
     if(ftd->q) xaccFreeQuery(ftd->q);
@@ -365,23 +516,17 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
   action_match_text = 
     gtk_entry_get_text(GTK_ENTRY(ftd->action_entry));
   
-  if(g_list_length(selected_accounts)) {
-    /* build the acclist from the widget GList */
-    acclist = g_new0(Account *, 
-                     g_list_length(selected_accounts) + 1);
-    i = 0;
-    for(cur=selected_accounts; cur; cur=cur->next) {
-      acclist[i] = cur->data;
-      i++;
-    }
-    acclist[i] = NULL;
-    
-    xaccQueryAddAccountMatch(q, acclist,
+  /* tag */
+  tag_match_text = 
+    gtk_entry_get_text(GTK_ENTRY(ftd->tag_entry));
+  
+  if(selected_accounts) {
+    xaccQueryAddAccountMatch(q, selected_accounts,
                              gnc_option_menu_get_active
-                             (ftd->match_accounts_picker),
+                             (ftd->match_accounts_picker) + 1,
                              QUERY_AND);
   }                             
-  
+
   if(strlen(descript_match_text)) {    
     xaccQueryAddDescriptionMatch(q, descript_match_text, 
                                  gtk_toggle_button_get_active
@@ -392,7 +537,6 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
                                   (ftd->description_regexp_toggle)),
                                  QUERY_AND);
   }
-  
 
   if(strlen(number_match_text)) {    
     xaccQueryAddNumberMatch(q, number_match_text, 
@@ -404,30 +548,26 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
                              (ftd->number_regexp_toggle)),
                             QUERY_AND);
   }
-  
 
-  amt_temp = 
-    (double)gtk_spin_button_get_value_as_float
-    (GTK_SPIN_BUTTON(ftd->amount_entry));
-  
-  amt_type = gnc_option_menu_get_active(ftd->credit_debit_picker);
-  
-  if((amt_temp > 0.00001) || (amt_type != 0)) {
-    xaccQueryAddAmountMatch(q, 
+  amt_temp = gnc_amount_edit_get_damount (GNC_AMOUNT_EDIT (ftd->amount_edit));
+  amt_type = gnc_option_menu_get_active(ftd->credit_debit_picker) + 1;
+
+  if((amt_temp > 0.00001) || (amt_type != AMT_SGN_MATCH_EITHER)) {
+    DxaccQueryAddAmountMatch(q, 
                             amt_temp,
                             amt_type,
                             gnc_option_menu_get_active
-                            (ftd->amount_comp_picker),
+                            (ftd->amount_comp_picker) + 1,
                             QUERY_AND);
   }
-  
+
   use_start_date = 
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ftd->date_start_toggle));
   use_end_date = 
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ftd->date_end_toggle));
 
   start_date = gnc_date_edit_get_date(GNC_DATE_EDIT(ftd->date_start_entry));
-  end_date   = gnc_date_edit_get_date(GNC_DATE_EDIT(ftd->date_end_entry));
+  end_date   = gnc_date_edit_get_date_end(GNC_DATE_EDIT(ftd->date_end_entry));
 
   if(use_start_date || use_end_date) {
     xaccQueryAddDateMatchTT(q, 
@@ -435,7 +575,7 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
                             use_end_date, end_date,
                             QUERY_AND);
   }
-  
+
   if(strlen(memo_match_text)) {    
     xaccQueryAddMemoMatch(q, memo_match_text, 
                           gtk_toggle_button_get_active
@@ -445,27 +585,21 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
                           QUERY_AND);    
   }
 
-  amt_temp = 
-    (double)gtk_spin_button_get_value_as_float
-    (GTK_SPIN_BUTTON(ftd->price_entry));
-  amt_type = 
-    gnc_option_menu_get_active(ftd->price_comp_picker);
+  amt_temp = gnc_amount_edit_get_damount (GNC_AMOUNT_EDIT (ftd->price_edit));
+  amt_type = gnc_option_menu_get_active(ftd->price_comp_picker) + 1;
 
-  if((amt_temp > 0.00001) || (amt_type != 0)) {
-    xaccQueryAddSharePriceMatch(q, 
+  if((amt_temp > 0.00001) || (amt_type != AMT_MATCH_ATLEAST)) {
+    DxaccQueryAddSharePriceMatch(q, 
                                 amt_temp,
                                 amt_type,
                                 QUERY_AND);
   }
-  
-  amt_temp = 
-    (double)gtk_spin_button_get_value_as_float
-    (GTK_SPIN_BUTTON(ftd->shares_entry));
-  amt_type = 
-    gnc_option_menu_get_active(ftd->shares_comp_picker);
-  
-  if((amt_temp > 0.00001) || (amt_type != 0)) {
-    xaccQueryAddSharesMatch(q, 
+
+  amt_temp = gnc_amount_edit_get_damount (GNC_AMOUNT_EDIT (ftd->shares_edit));
+  amt_type = gnc_option_menu_get_active(ftd->shares_comp_picker) + 1;
+
+  if((amt_temp > 0.00001) || (amt_type != AMT_MATCH_ATLEAST)) {
+    DxaccQueryAddSharesMatch(q, 
                             amt_temp,
                             amt_type,
                             QUERY_AND);
@@ -497,37 +631,57 @@ gnc_ui_find_transactions_dialog_ok_cb(GtkButton * button,
     xaccQueryAddClearedMatch(q, how, QUERY_AND);
   }
 
-  if(!ftd->ledger) {
-    new_ledger = TRUE;
-    ftd->ledger = xaccLedgerDisplayGeneral(NULL, NULL, SEARCH_LEDGER);
-    xaccFreeQuery(ftd->ledger->query);  
+  b_balanced = gtk_toggle_button_get_active
+    (GTK_TOGGLE_BUTTON(ftd->balance_balanced_toggle));
+  b_not_balanced = gtk_toggle_button_get_active
+    (GTK_TOGGLE_BUTTON(ftd->balance_not_balanced_toggle));
+
+  if(b_balanced || b_not_balanced) {
+    balance_match_t how = 0;
+    if(b_balanced)     how = how | BALANCE_BALANCED;
+    if(b_not_balanced) how = how | BALANCE_UNBALANCED;
+    xaccQueryAddBalanceMatch(q, how, QUERY_AND);
   }
-  
+
   switch(search_type) {
   case 0:
-    ftd->ledger->query = q;
+    new_q = q;
     break;
   case 1:    
-    ftd->ledger->query = xaccQueryMerge(ftd->q, q, QUERY_AND);
+    new_q = xaccQueryMerge(ftd->q, q, QUERY_AND);
     xaccFreeQuery(q);
     break;
   case 2:
-    ftd->ledger->query = xaccQueryMerge(ftd->q, q, QUERY_OR);
+    new_q = xaccQueryMerge(ftd->q, q, QUERY_OR);
     xaccFreeQuery(q);
     break;
   case 3:
     q2 = xaccQueryInvert(q);
-    ftd->ledger->query = xaccQueryMerge(ftd->q, q2, QUERY_AND);
+    new_q = xaccQueryMerge(ftd->q, q2, QUERY_AND);
     xaccFreeQuery(q2);
     xaccFreeQuery(q);
     break;
+  default:
+    PERR ("bad search type: %d", search_type);
+    new_q = q;
+    break;
   }
 
-  ftd->ledger->dirty = 1;
-  xaccLedgerDisplayRefresh(ftd->ledger);
-  if (new_ledger) regWindowLedger(ftd->ledger);
-  
+  ledger = xaccFindGeneralLedgerByQuery (ftd->ledger_q);
+  if(!ledger) {
+    new_ledger = TRUE;
+    ledger = xaccLedgerDisplayQuery (new_q, SEARCH_LEDGER,
+                                     REG_STYLE_JOURNAL);
+  }
+  else
+    xaccLedgerDisplaySetQuery (ledger, new_q);
+
+  xaccFreeQuery (new_q);
+
+  xaccLedgerDisplayRefresh(ledger);
+
+  if (new_ledger)
+    regWindowLedger(ledger);
+
   gnc_ui_find_transactions_dialog_destroy(ftd);
 }
-
-
