@@ -1,4 +1,6 @@
 /*
+ * gnc-dateedit.c -- Date editor widget
+ *
  * Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation
  * All rights reserved.
  *
@@ -28,10 +30,11 @@
  * Date editor widget
  *
  * Authors: Miguel de Icaza
- *          Dave Peticolas <peticola@cs.ucdavis.edu>
+ *          Dave Peticolas <dave@krondo.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h> /* atoi */
 #include <ctype.h> /* isdigit */
@@ -39,6 +42,7 @@
 #include <time.h>
 
 #include "gnc-dateedit.h"
+#include "messages.h"
 #include "date.h"
 
 
@@ -58,6 +62,11 @@ static void gnc_date_edit_forall       (GtkContainer       *container,
                                         gboolean	    include_internals,
                                         GtkCallback	    callback,
                                         gpointer	    callbabck_data);
+static struct tm gnc_date_edit_get_date_internal (GNCDateEdit *gde);
+static int date_accel_key_press(GtkWidget *widget,
+                                GdkEventKey *event,
+                                gpointer data);
+
 
 static GtkHBoxClass *parent_class;
 
@@ -89,6 +98,35 @@ gnc_date_edit_get_type (void)
 	return date_edit_type;
 }
 
+
+static char *
+gnc_strtok_r (char *s, const char *delim, char **save_ptr)
+{
+  char *token;
+
+  if (s == NULL)
+    s = *save_ptr;
+
+  /* Scan leading delimiters.  */
+  s += strspn (s, delim);
+  if (*s == '\0')
+    return NULL;
+
+  /* Find the end of the token.  */
+  token = s;
+  s = strpbrk (token, delim);
+  if (s == NULL)
+    /* This token finishes the string.  */
+    *save_ptr = strchr (token, '\0');
+  else
+    {
+      /* Terminate the token and make *SAVE_PTR point past it.  */
+      *s = '\0';
+      *save_ptr = s + 1;
+    }
+  return token;
+}
+
 static void
 hide_popup (GNCDateEdit *gde)
 {
@@ -133,7 +171,7 @@ key_press_popup (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	GNCDateEdit *gde;
 
 	if (event->keyval != GDK_Escape)
-		return FALSE;
+		return date_accel_key_press(widget, event, data);
 
 	gde = data;
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
@@ -219,8 +257,20 @@ select_clicked (GtkWidget *widget, GNCDateEdit *gde)
         mtm.tm_hour = 0;
         mtm.tm_isdst = -1;
 
-        mktime(&mtm); /* normalize */
+        if (mktime (&mtm) == -1)
+        {
+                time_t secs = time (NULL);
 
+                mtm = *localtime (&secs);
+                mtm.tm_sec = 0;
+                mtm.tm_min = 0;
+                mtm.tm_hour = 0;
+                mtm.tm_isdst = -1;
+
+                gnc_date_edit_set_time (gde, mktime (&mtm));
+        }
+
+        gtk_calendar_select_day (GTK_CALENDAR (gde->calendar), 1);
 	gtk_calendar_select_month (GTK_CALENDAR (gde->calendar), mtm.tm_mon,
                                    1900 + mtm.tm_year);
         gtk_calendar_select_day (GTK_CALENDAR (gde->calendar), mtm.tm_mday);
@@ -398,22 +448,24 @@ gnc_date_edit_destroy (GtkObject *object)
 
 static void
 gnc_date_edit_forall (GtkContainer *container, gboolean include_internals,
-			GtkCallback callback, gpointer callback_data)
+                      GtkCallback callback, gpointer callback_data)
 {
 	g_return_if_fail (container != NULL);
 	g_return_if_fail (GNC_IS_DATE_EDIT (container));
 	g_return_if_fail (callback != NULL);
 
-	/* Let GtkBox handle things only if the internal widgets need to be
-	 * poked.
-	 */
-	if (include_internals)
-		if (GTK_CONTAINER_CLASS (parent_class)->forall)
-			(* GTK_CONTAINER_CLASS (parent_class)->forall)
-                                (container,
-                                 include_internals,
-                                 callback,
-                                 callback_data);
+	/* Let GtkBox handle things only if the internal widgets need
+	 * to be poked.  */
+	if (!include_internals)
+                return;
+
+        if (!GTK_CONTAINER_CLASS (parent_class)->forall)
+                return;
+
+        GTK_CONTAINER_CLASS (parent_class)->forall (container,
+                                                    include_internals,
+                                                    callback,
+                                                    callback_data);
 }
 
 /**
@@ -473,6 +525,177 @@ gnc_date_edit_set_popup_range (GNCDateEdit *gde, int low_hour, int up_hour)
         fill_time_popup(NULL, gde);
 }
 
+/* This code should be kept in sync with src/register/datecell.c */
+static int
+date_accel_key_press(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	GNCDateEdit *gde = data;
+        char *string;
+        struct tm tm;
+        GDate gdate;
+
+        switch (event->keyval) {
+                case GDK_plus:
+                case GDK_KP_Add:
+                case GDK_equal:
+                case GDK_KP_Equal:
+                case GDK_underscore:
+                case GDK_minus:
+                case GDK_KP_Subtract:
+                case GDK_bracketright:
+                case GDK_braceright:
+                case GDK_bracketleft:
+                case GDK_braceleft:
+                case GDK_M:
+                case GDK_m:
+                case GDK_H:
+                case GDK_h:
+                case GDK_Y:
+                case GDK_y:
+                case GDK_R:
+                case GDK_r:
+                case GDK_T:
+                case GDK_t:
+                        break;
+                default:
+                        return FALSE;
+        }
+
+        string = gtk_entry_get_text (GTK_ENTRY (widget));
+
+        gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+
+        tm = gnc_date_edit_get_date_internal (gde);
+
+        g_date_set_dmy (&gdate, tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900);
+
+        switch (event->keyval)
+        {
+                case GDK_KP_Add:
+                case GDK_plus:
+                case GDK_equal:
+                        if (event->state & GDK_SHIFT_MASK)
+                                g_date_add_days (&gdate, 7);
+                        else if (event->state & GDK_MOD1_MASK)
+                                g_date_add_months (&gdate, 1);
+                        else if (event->state & GDK_CONTROL_MASK)
+                                g_date_add_years (&gdate, 1);
+                        else
+                                g_date_add_days (&gdate, 1);
+                        break;
+
+                case GDK_minus:
+                        if ((strlen (string) != 0) && (dateSeparator () == '-'))
+                        {
+                                int i;
+                                int len;
+                                int count;
+
+                                len = strlen (string);
+                                /* rough check for existing date */
+                                for (i = count = 0; i < len; i++)
+                                {
+                                        if (string[i] == '-')
+                                                count++;
+                                }
+
+                                if (count < 2)
+                                        return FALSE;
+                        }
+
+                        /* fall through */
+                case GDK_KP_Subtract:
+                case GDK_underscore:
+                        if (event->state & GDK_SHIFT_MASK)
+                                g_date_subtract_days (&gdate, 7);
+                        else if (event->state & GDK_MOD1_MASK)
+                                g_date_subtract_months (&gdate, 1);
+                        else if (event->state & GDK_CONTROL_MASK)
+                                g_date_subtract_years (&gdate, 1);
+                        else
+                                g_date_subtract_days (&gdate, 1);
+                        break;
+
+                case GDK_braceright:
+                case GDK_bracketright:
+                        /* increment month */
+                        g_date_add_months (&gdate, 1);
+                        break;
+
+                case GDK_braceleft:
+                case GDK_bracketleft:
+                        /* decrement month */
+                        g_date_subtract_months (&gdate, 1);
+                        break;
+
+                case GDK_M:
+                case GDK_m:
+                        /* beginning of month */
+                        g_date_set_day (&gdate, 1);
+                        break;
+
+                case GDK_H:
+                case GDK_h:
+                        /* end of month */
+                        g_date_set_day (&gdate, 1);
+                        g_date_add_months (&gdate, 1);
+                        g_date_subtract_days (&gdate, 1);
+                        break;
+
+                case GDK_Y:
+                case GDK_y:
+                        /* beginning of year */
+                        g_date_set_day (&gdate, 1);
+                        g_date_set_month (&gdate, 1);
+                        break;
+
+                case GDK_R:
+                case GDK_r:
+                        /* end of year */
+                        g_date_set_day (&gdate, 1);
+                        g_date_set_month (&gdate, 1);
+                        g_date_add_years (&gdate, 1);
+                        g_date_subtract_days (&gdate, 1);
+                        break;
+
+                case GDK_T:
+                case GDK_t:
+                        {
+                                /* today */
+                                GTime gtime;
+
+                                gtime = time (NULL);
+                                g_date_set_time (&gdate, gtime);
+                                break;
+                        }
+
+                default:
+                        return FALSE;
+        }
+
+        g_date_to_struct_tm (&gdate, &tm);
+
+        if (mktime (&tm) == -1)
+        {
+                time_t secs = time (NULL);
+
+                tm = *localtime (&secs);
+                tm.tm_sec = 0;
+                tm.tm_min = 0;
+                tm.tm_hour = 0;
+                tm.tm_isdst = -1;
+        }
+
+        gnc_date_edit_set_time (gde, mktime (&tm));
+
+        gtk_calendar_select_day (GTK_CALENDAR (gde->calendar), 1);
+	gtk_calendar_select_month (GTK_CALENDAR (gde->calendar), tm.tm_mon,
+                                   1900 + tm.tm_year);
+        gtk_calendar_select_day (GTK_CALENDAR (gde->calendar), tm.tm_mday);
+
+        return TRUE;
+}
+
 static void
 create_children (GNCDateEdit *gde)
 {
@@ -484,7 +707,9 @@ create_children (GNCDateEdit *gde)
 	gtk_widget_set_usize (gde->date_entry, 90, 0);
 	gtk_box_pack_start (GTK_BOX (gde), gde->date_entry, TRUE, TRUE, 0);
 	gtk_widget_show (gde->date_entry);
-	
+	gtk_signal_connect (GTK_OBJECT (gde->date_entry), "key_press_event",
+			    GTK_SIGNAL_FUNC(date_accel_key_press), gde);
+
 	gde->date_button = gtk_button_new ();
 	gtk_signal_connect (GTK_OBJECT (gde->date_button), "clicked",
 			    GTK_SIGNAL_FUNC (select_clicked), gde);
@@ -607,14 +832,8 @@ gnc_date_edit_new_flags (time_t the_time, GNCDateEditFlags flags)
 	return GTK_WIDGET (gde);
 }
 
-/**
- * gnc_date_edit_get_date:
- * @gde: The GNCDateEdit widget
- *
- * Returns the time entered in the GNCDateEdit widget
- */
-time_t
-gnc_date_edit_get_date (GNCDateEdit *gde)
+static struct tm
+gnc_date_edit_get_date_internal (GNCDateEdit *gde)
 {
 	struct tm tm = {0};
 	char *str, *flags = NULL;
@@ -639,18 +858,20 @@ gnc_date_edit_get_date (GNCDateEdit *gde)
 
 		str = g_strdup (gtk_entry_get_text
                                 (GTK_ENTRY (gde->time_entry)));
-		temp = strtok_r (str, ": ", &tokp);
+		temp = gnc_strtok_r (str, ": ", &tokp);
 		if (temp) {
 			tm.tm_hour = atoi (temp);
-			temp = strtok_r (NULL, ": ", &tokp);
+			temp = gnc_strtok_r (NULL, ": ", &tokp);
 			if (temp) {
 				if (isdigit (*temp)) {
 					tm.tm_min = atoi (temp);
-					flags = strtok_r (NULL, ": ", &tokp);
+					flags = gnc_strtok_r (NULL, ": ",
+                                                              &tokp);
 					if (flags && isdigit (*flags)) {
 						tm.tm_sec = atoi (flags);
-						flags = strtok_r (NULL, ": ",
-                                                                  &tokp);
+						flags = gnc_strtok_r (NULL,
+                                                                      ": ",
+                                                                      &tokp);
 					}
 				} else
 					flags = temp;
@@ -663,9 +884,34 @@ gnc_date_edit_get_date (GNCDateEdit *gde)
 		}
 		g_free (str);
 	}
+        else
+        {
+                tm.tm_hour = 0;
+                tm.tm_min  = 0;
+                tm.tm_sec  = 0;
+        }
 
 	tm.tm_isdst = -1;
 
+        return tm;
+}
+
+/**
+ * gnc_date_edit_get_date:
+ * @gde: The GNCDateEdit widget
+ *
+ * Returns the time entered in the GNCDateEdit widget
+ */
+time_t
+gnc_date_edit_get_date (GNCDateEdit *gde)
+{
+ 	struct tm tm;
+
+        g_return_val_if_fail (gde != NULL, 0);
+        g_return_val_if_fail (GNC_IS_DATE_EDIT (gde), 0);
+
+        tm = gnc_date_edit_get_date_internal (gde);
+
         if (mktime (&tm) == -1)
         {
                 time_t secs = time (NULL);
@@ -680,6 +926,41 @@ gnc_date_edit_get_date (GNCDateEdit *gde)
 	return mktime (&tm);
 }
 
+/**
+ * gnc_date_edit_get_date_end:
+ * @gde: The GNCDateEdit widget
+ *
+ * Returns the date entered in the GNCDateEdit widget,
+ * but with the time adjusted to the end of the day.
+ */
+time_t
+gnc_date_edit_get_date_end (GNCDateEdit *gde)
+{
+ 	struct tm tm;
+
+        g_return_val_if_fail (gde != NULL, 0);
+        g_return_val_if_fail (GNC_IS_DATE_EDIT (gde), 0);
+
+        tm = gnc_date_edit_get_date_internal (gde);
+
+        tm.tm_hour = 23;
+        tm.tm_min  = 59;
+        tm.tm_sec  = 59;
+
+        if (mktime (&tm) == -1)
+        {
+                time_t secs = time (NULL);
+
+                tm = *localtime (&secs);
+                tm.tm_sec = 23;
+                tm.tm_min = 59;
+                tm.tm_hour = 59;
+                tm.tm_isdst = -1;
+        }
+
+	return mktime (&tm);
+}
+
 /**
  * gnc_date_edit_set_flags:
  * @gde: The date editor widget whose flags should be changed.
