@@ -25,6 +25,9 @@
 
 #include <gtk/gtkclist.h>
 
+#include "Query.h"
+#include "gnc-numeric.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif				/* __cplusplus */
@@ -58,11 +61,10 @@ struct _GNCReconcileList
 
   gboolean no_toggle;
   gboolean always_unselect;
+  gboolean first_fill;
 
   GHashTable *reconciled;
 
-  GtkStyle *reconciled_style;
-
   Account *account;
   Query *query;
 };
@@ -73,6 +75,8 @@ struct _GNCReconcileListClass
 
   void (*toggle_reconciled) (GNCReconcileList *list,
 			     Split            *split);
+  void (*double_click_split) (GNCReconcileList *list,
+                              Split            *split);
 };
 
 /***********************************************************
@@ -93,9 +97,11 @@ Split * gnc_reconcile_list_get_current_split(GNCReconcileList *list);
 
 void gnc_reconcile_list_refresh (GNCReconcileList *list);
 
-double gnc_reconcile_list_reconciled_balance(GNCReconcileList *list);
+gnc_numeric gnc_reconcile_list_reconciled_balance(GNCReconcileList *list);
 
-void gnc_reconcile_list_commit(GNCReconcileList *list, time_t date);
+void gnc_reconcile_list_commit (GNCReconcileList *list, time_t date);
+
+void gnc_reconcile_list_postpone (GNCReconcileList *list);
 
 void gnc_reconcile_list_unselect_all(GNCReconcileList *list);
 
