@@ -1,4 +1,6 @@
 /********************************************************************\
+ * textcell.h -- simple plain-ascii text display/input cell         *
+ *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -29,18 +31,19 @@
  *
  * HISTORY:
  * Copyright (c) 1998 Linas Vepstas
+ * Copyright (c) 2000 Dave Peticolas
  */
 
-#ifndef __XACC_TEXT_CELL_H__
-#define __XACC_TEXT_CELL_H__
+#ifndef __TEXT_CELL_H__
+#define __TEXT_CELL_H__
 
 #include "basiccell.h"
 
 /* installs a callback to handle text recording */
 BasicCell * xaccMallocTextCell (void);
-void         xaccInitTextCell (BasicCell *);
-void         xaccDestroyTextCell (BasicCell *);
+void        xaccInitTextCell (BasicCell *cell);
+void        xaccDestroyTextCell (BasicCell *cell);
 
-#endif /* __XACC_TEXT_CELL_H__ */
+#endif /* __TEXT_CELL_H__ */
 
 /* --------------- end of file ---------------------- */
