@@ -18,16 +18,16 @@
 ;; Boston, MA  02111-1307,  USA       gnu@gnu.org
 
 (define (gnc:make-option
-	 ;; The category of this option
+         ;; The category of this option
          section
          name
-	 ;; The sort-tag determines the relative ordering of options in
-	 ;; this category. It is used by the gui for display.
+         ;; The sort-tag determines the relative ordering of options in
+         ;; this category. It is used by the gui for display.
          sort-tag
          type
          documentation-string
          getter
-	 ;; The setter is responsible for ensuring that the value is valid.
+         ;; The setter is responsible for ensuring that the value is valid.
          setter
          default-getter
          ;; Restore form generator should generate an ascii representation
@@ -35,32 +35,59 @@
          ;; option. The function should restore the option to the original
          ;; value.
          generate-restore-form
-	 ;; Validation func should accept a value and return (#t value)
-	 ;; on success, and (#f "failure-message") on failure. If #t,
-	 ;; the supplied value will be used by the gui to set the option.
+         ;; Validation func should accept a value and return (#t value)
+         ;; on success, and (#f "failure-message") on failure. If #t,
+         ;; the supplied value will be used by the gui to set the option.
          value-validator
-	 option-data
-         ;; This function should return a list of all the strings in the
-         ;; option other than the section, name, and documentation-string
-         ;; that might be displayed to the user (and thus should be
+	 ;;; free-form storage depending on type.
+         option-data 
+	 ;; If this is a "multiple choice" type of option,
+	 ;; this should be a vector of the following five functions
+	 ;; one taking no arguments giving the number of choices
+         ;; one taking one argument, a non-negative integer, that
+         ;; returns the scheme value (usually a symbol) matching the
+         ;; nth choice
+         ;;
+	 ;; one taking one argument, a non-negative integer,
+	 ;; that returns the string matching the nth choice
+         ;;
+	 ;; the third takes one argument and returns the description
+	 ;; containing the nth choice
+	 ;;
+	 ;; the fourth giving a possible value and returning the index
+	 ;; if an option doesn't use these,  this should just be a #f
+	 option-data-fns
+         ;; This function should return a list of all the strings
+         ;; in the option other than the section, name, (define
+         ;; (list-lookup list item) and documentation-string that
+         ;; might be displayed to the user (and thus should be
          ;; translated).
-         strings-getter)
+         strings-getter
+         ;; This function will be called when the GUI representation
+         ;; of the option is changed.  This will normally occur before
+         ;; the setter is called, because setters are only called when
+         ;; the user selects "OK" or "Apply".  Therefore, this
+         ;; callback shouldn't be used to make changes to the actual
+         ;; options database.
+         option-widget-changed-proc)
   (let ((changed-callback #f))
     (vector section
             name
             sort-tag
             type
             documentation-string
-            getter
-            (lambda args
+            getter            
+	    (lambda args
               (apply setter args)
               (if changed-callback (changed-callback)))
             default-getter
             generate-restore-form
             value-validator
             option-data
+	    option-data-fns
             (lambda (callback) (set! changed-callback callback))
-            strings-getter)))
+            strings-getter
+            option-widget-changed-proc)))
 
 (define (gnc:option-section option)
   (vector-ref option 0))
@@ -80,20 +107,54 @@
   (vector-ref option 7))
 (define (gnc:option-generate-restore-form option)
   (vector-ref option 8))
-(define (gnc:option-value-validator option)
+(define (gnc:option-value-validator option)  
   (vector-ref option 9))
 (define (gnc:option-data option)
-  (vector-ref option 10))
+    (vector-ref option 10))
+(define (gnc:option-data-fns option)
+  (vector-ref option 11))
+
 (define (gnc:option-set-changed-callback option callback)
-  (let ((cb-setter (vector-ref option 11)))
+ (let ((cb-setter (vector-ref option 12)))
     (cb-setter callback)))
 (define (gnc:option-strings-getter option)
-  (vector-ref option 12))
+  (vector-ref option 13))
+(define (gnc:option-widget-changed-proc option)
+  (vector-ref option 14))
 
 (define (gnc:option-value option)
   (let ((getter (gnc:option-getter option)))
     (getter)))
 
+(define (gnc:option-set-value option value)
+  (let ((setter (gnc:option-setter option)))
+    (setter value)))
+
+(define (gnc:option-index-get-name option index)
+  (let* ((option-data-fns (gnc:option-data-fns option))
+	 (name-fn (vector-ref option-data-fns 2)))
+    (name-fn index)))
+
+(define (gnc:option-index-get-description option index)
+  (let* ((option-data-fns (gnc:option-data-fns option))
+	 (name-fn (vector-ref option-data-fns 3)))
+    (name-fn index)))
+
+(define (gnc:option-index-get-value option index)
+  (let* ((option-data-fns (gnc:option-data-fns option))
+	 (name-fn (vector-ref option-data-fns 1)))
+    (name-fn index)))
+
+(define (gnc:option-value-get-index option value)
+  (let* ((option-data-fns (gnc:option-data-fns option))
+	 (name-fn (vector-ref option-data-fns 4)))
+    (name-fn value)))
+
+(define (gnc:option-number-of-indices option)
+  (let* ((option-data-fns (gnc:option-data-fns option))
+	 (name-fn (vector-ref option-data-fns 0)))
+    (name-fn)))
+
 (define (gnc:option-default-value option)
   (let ((getter (gnc:option-default-getter option)))
     (getter)))
@@ -127,31 +188,9 @@
      (lambda (x)
        (cond ((string? x)(list #t x))
              (else (list #f "string-option: not a string"))))
-     #f #f )))
+     #f #f #f #f)))
 
-;; currency options use a specialized widget for entering currencies
-;; in the GUI implementation. Currencies must be 3-letter ISO codes
-;; represented as strings.
-(define (gnc:make-currency-option
-	 section
-	 name
-	 sort-tag
-         documentation-string
-	 default-value)
-  (let* ((value default-value)
-         (value->string (lambda () (gnc:value->string value))))
-    (gnc:make-option
-     section name sort-tag 'currency documentation-string
-     (lambda () value)
-     (lambda (x) (set! value x))
-     (lambda () default-value)
-     (gnc:restore-form-generator value->string)
-     (lambda (x)
-       (cond ((string? x)(list #t x))
-             (else (list #f "currency-option: not a currency code"))))
-     #f #f )))
-
-(define (gnc:make-simple-boolean-option
+(define (gnc:make-text-option
 	 section
 	 name
 	 sort-tag
@@ -160,30 +199,208 @@
   (let* ((value default-value)
          (value->string (lambda () (gnc:value->string value))))
     (gnc:make-option
-     section name sort-tag 'boolean documentation-string
+     section name sort-tag 'text documentation-string
      (lambda () value)
      (lambda (x) (set! value x))
      (lambda () default-value)
      (gnc:restore-form-generator value->string)
      (lambda (x)
-       (if (boolean? x)
-           (list #t x)
-           (list #f "boolean-option: not a boolean")))
-     #f #f )))
+       (cond ((string? x)(list #t x))
+             (else (list #f "text-option: not a string"))))
+     #f #f #f #f)))
+
+;;; font options store fonts as strings a la the X Logical
+;;; Font Description. You should always provide a default
+;;; value, as currently there seems to be no way to go from
+;;; an actual font to a logical font description, and thus
+;;; there is no way for the gui to pick a default value.
+
+(define (gnc:make-font-option
+	 section
+	 name
+	 sort-tag
+	 documentation-string
+	 default-value)
+  (let* ((value default-value)
+         (value->string (lambda () (gnc:value->string value))))
+    (gnc:make-option
+     section
+     name
+     sort-tag
+     'font
+     documentation-string
+     (lambda () value)
+     (lambda (x) (set! value x))
+     (lambda () default-value)
+     (gnc:restore-form-generator value->string)     
+     (lambda (x)
+       (cond ((string? x)(list #t x))
+             (else (list #f "font-option: not a string"))))
+     #f #f #f #f)))
+
+;; currency options use a specialized widget for entering currencies
+;; in the GUI implementation.
+(define (gnc:make-currency-option
+	 section
+	 name
+	 sort-tag
+         documentation-string
+ 	 default-value)
+
+  (define (currency->scm currency)
+    (if (string? currency)
+        currency
+        (gnc:commodity-get-mnemonic currency)))
+
+  (define (scm->currency currency)
+    (if (string? currency)
+        (gnc:commodity-table-lookup
+         (gnc:engine-commodities) GNC_COMMODITY_NS_ISO currency)
+        currency))
+
+   (let* ((value (currency->scm default-value))
+          (value->string (lambda () (gnc:value->string value))))
+     (gnc:make-option
+      section name sort-tag 'currency documentation-string
+      (lambda ()  (scm->currency value))
+      (lambda (x) (set! value (currency->scm x)))
+      (lambda ()  (scm->currency default-value))
+      (gnc:restore-form-generator value->string)
+      (lambda (x) (list #t x))
+      #f #f #f #f)))
+
+;; commodity options use a specialized widget for entering commodities
+;; in the GUI implementation.
+(define (gnc:make-commodity-option
+	 section
+	 name
+	 sort-tag
+         documentation-string
+ 	 default-value)
+
+  (define (commodity->scm commodity)
+    (list 'commodity-scm
+          (gnc:commodity-get-namespace commodity)
+          (gnc:commodity-get-mnemonic commodity)))
+
+  (define (scm->commodity scm)
+    (gnc:commodity-table-lookup
+     (gnc:engine-commodities) (cadr scm) (caddr scm)))
+
+   (let* ((value (commodity->scm default-value))
+          (value->string (lambda ()
+                           (string-append "'" (gnc:value->string value)))))
+     (gnc:make-option
+      section name sort-tag 'commodity documentation-string
+      (lambda () (scm->commodity value))
+      (lambda (x) (if (and (pair? x) (eqv? (car x) 'commodity-scm))
+                      (set! value x)
+                      (set! value (commodity->scm x))))
+      (lambda () default-value)
+      (gnc:restore-form-generator value->string)
+      (lambda (x) (list #t x))
+      #f #f #f #f)))
+
+
+(define (gnc:make-simple-boolean-option
+ 	 section
+ 	 name
+ 	 sort-tag
+ 	 documentation-string
+ 	 default-value)
+  (gnc:make-complex-boolean-option section
+				   name
+				   sort-tag
+				   documentation-string
+				   default-value
+				   #f 
+				   #f))
+
+;; Complex boolean options are the same as simple boolean options (see
+;; above), with the addition of two function arguments. (If both of
+;; them are #f, you have exactly a simple-boolean-option.) Both
+;; functions should expect one boolean argument.  When the option's
+;; value is changed, the function option-widget-changed-cb will be
+;; called with the new option value at the time that the GUI widget
+;; representing the option is changed, and the function
+;; setter-function-called-cb will be called when the option's setter
+;; is called (that is, when the user selects "OK" or "Apply").
+
+;; The option-widget-changed-cb is tested for procedurehood before
+;; it is called, so it is not validated to be a procedure here.
+;; However, since there could be an option-widget-changed-cb but not
+;; a setter-function-called-cb, the procedurehood of the
+;; setter-function-called-cb is checked here.
+(define (gnc:make-complex-boolean-option
+ 	 section
+ 	 name
+ 	 sort-tag
+ 	 documentation-string
+ 	 default-value
+         setter-function-called-cb
+         option-widget-changed-cb)
+   (let* ((value default-value)
+          (value->string (lambda () (gnc:value->string value))))
+     (gnc:make-option
+      section name sort-tag 'boolean documentation-string
+      (lambda () value)
+      (lambda (x) (set! value x)
+                  (if (procedure? setter-function-called-cb)
+                      (setter-function-called-cb x)))
+      (lambda () default-value)
+      (gnc:restore-form-generator value->string)
+      (lambda (x)
+        (if (boolean? x)
+            (list #t x)
+            (list #f "boolean-option: not a boolean")))
+      #f #f #f (and option-widget-changed-cb
+		    (lambda (x) (option-widget-changed-cb x))))))
+
+
+(define (gnc:make-pixmap-option 
+         section name sort-tag doc-string
+         default-value)
+  (let* ((value default-value))
+    (gnc:make-option
+     section name sort-tag 'pixmap doc-string
+     (lambda ()  value)
+     (lambda (x) (set! value x))
+     (lambda () default-value)
+     (gnc:restore-form-generator  (lambda () (gnc:value->string value)))
+     (lambda (x)
+       (if (string? x)
+           (begin 
+             (list #t x))
+           (begin 
+             (list #f "pixmap-option: not a string"))))
+     #f #f #f #f)))
+
+;; subtype should be on of 'relative 'absolute or 'both
+;; relative-date-list should be the list of relative dates permitted
+;; gnc:all-relative-dates contains a list of all relative dates.
 
-;; date options use the option-data as a boolean value. If true,
-;; the gui should allow the time to be entered as well.
 (define (gnc:make-date-option
          section
          name
-         sort-tag
+         sort-tag 
          documentation-string
          default-getter
-         show-time)
-
+         show-time
+	 subtype
+	 relative-date-list)
   (define (date-legal date)
-    (and (pair? date) (exact? (car date)) (exact? (cdr date))))
-
+    (and (pair? date) 
+	 (or 
+	  (and (eq? 'relative (car date)) (symbol? (cdr date)))
+          (and (eq? 'absolute (car date)) 
+               (pair? (cdr date)) 
+               (exact? (cadr date)) 
+               (exact? (cddr date))))))
+  (define (list-lookup list item)
+    (cond
+     ((null? list) #f)
+     ((eq? item (car list)) 0)
+     (else (+ 1 (list-lookup (cdr list) item)))))
   (let* ((value (default-getter))
          (value->string (lambda ()
                           (string-append "'" (gnc:value->string value)))))
@@ -193,16 +410,57 @@
      (lambda (date)
        (if (date-legal date)
            (set! value date)
-           (gnc:error "Illegal date value set")))
+           (gnc:error "Illegal date value set:" date)))
      default-getter
      (gnc:restore-form-generator value->string)
      (lambda (date)
        (if (date-legal date)
            (list #t date)
            (list #f "date-option: illegal date")))
-     show-time #f)))
+     (vector subtype show-time relative-date-list) 
+     (vector (lambda () (length relative-date-list))
+             (lambda (x) (list-ref relative-date-list x))
+             (lambda (x) (gnc:get-relative-date-string
+                          (list-ref relative-date-list x)))
+             (lambda (x) (gnc:get-relative-date-desc
+                          (list-ref relative-date-list x)))
+             (lambda (x) (list-lookup relative-date-list x)))
+     #f
+     #f)))
 
-;; account-list options use the option-data as a boolean value.  If
+(define (gnc:get-rd-option-data-subtype option-data)
+  (vector-ref option-data 0))
+
+(define (gnc:get-rd-option-data-show-time option-data)
+  (vector-ref option-data 1))
+
+(define (gnc:get-rd-option-data-rd-list option-data)
+  (vector-ref option-data 2))
+
+(define (gnc:date-option-get-subtype option)
+  (if (eq? (gnc:option-type option) 'date)
+      (gnc:get-rd-option-data-subtype (gnc:option-data option))
+      (gnc:error "Not a date option")))
+
+(define (gnc:date-option-show-time? option)
+  (if (eq? (gnc:option-type option) 'date)
+      (gnc:get-rd-option-data-show-time (gnc:option-data option))
+      (gnc:error "Not a date option")))
+
+(define (gnc:date-option-value-type option-value)
+  (car option-value))
+
+(define (gnc:date-option-absolute-time option-value)
+  (if (eq? (car option-value) 'absolute)
+      (cdr option-value)
+      (gnc:get-absolute-from-relative-date (cdr option-value))))
+
+(define (gnc:date-option-relative-time option-value)
+  (if (eq? (car option-value) 'absolute)
+      #f
+      (cdr option-value)))
+
+;; account-list options use the option-data as a boolean value. If
 ;; true, the gui should allow the user to select multiple accounts.
 ;; Internally, values are always a list of guids. Externally, both
 ;; guids and account pointers may be used to set the value of the
@@ -220,23 +478,34 @@
     (if (string? item)
         item
         (gnc:account-get-guid item)))
+
   (define (convert-to-account item)
     (if (string? item)
         (gnc:account-lookup item)
         item))
 
-  (let ((option (map convert-to-guid (default-getter)))
-        (option-set #f)
-        (validator
-         (if (not value-validator)
-             (lambda (account-list) (list #t account-list))
-             value-validator)))
+  (let* ((option (map convert-to-guid (default-getter)))
+         (option-set #f)
+         (getter (lambda () (map convert-to-account
+                                 (if option-set
+                                     option
+                                     (default-getter)))))
+         (value->string (lambda ()
+                          (string-append
+                           "'" (gnc:value->string (if option-set option #f)))))
+         (validator
+          (if (not value-validator)
+              (lambda (account-list) (list #t account-list))
+              (lambda (account-list)
+                (value-validator (map convert-to-account account-list))))))
     (gnc:make-option
-     section name sort-tag 'account-list documentation-string
-     (lambda () (map convert-to-account (if option-set
-                                            option
-                                            (default-getter))))
+     section name sort-tag 'account-list documentation-string getter
      (lambda (account-list)
+       (if (not account-list) (set! account-list (default-getter)))
+       (set! account-list
+             (filter (lambda (x) (if (string? x)
+                                     (gnc:account-lookup x)
+                                     x)) account-list))
        (let* ((result (validator account-list))
               (valid (car result))
               (value (cadr result)))
@@ -246,9 +515,15 @@
                (set! option-set #t))
              (gnc:error "Illegal account list value set"))))
      (lambda () (map convert-to-account (default-getter)))
-     #f
+     (gnc:restore-form-generator value->string)
      validator
-     multiple-selection #f)))
+     multiple-selection #f #f #f)))
+
+(define (gnc:multichoice-list-lookup list item )
+  (cond
+   ((null? list) #f)
+   ((eq? item (vector-ref (car list) 0)) 0)
+   (else (+ 1 (gnc:multichoice-list-lookup (cdr list) item)))))
 
 ;; multichoice options use the option-data as a list of vectors.
 ;; Each vector contains a permissible value (scheme symbol), a
@@ -260,7 +535,34 @@
          documentation-string
          default-value
          ok-values)
+  (gnc:make-multichoice-callback-option section
+					name
+					sort-tag
+					documentation-string
+					default-value
+					ok-values
+					#f
+					#f))
 
+;; The multichoice-option with callback function is the same as the
+;; usual multichoice options (see above), with the addition of two
+;; function arguments. (If both of them are #f, you have exactly a
+;; multichoice-option.) Both functions should expect one argument.
+;; When the option's value is changed, the function
+;; option-widget-changed-cb will be called with the new option value
+;; at the time that the GUI widget representing the option is changed,
+;; and the function setter-function-called-cb will be called when the
+;; option's setter is called (that is, when the user selects "OK" or
+;; "Apply").
+(define (gnc:make-multichoice-callback-option
+         section
+         name
+         sort-tag
+         documentation-string
+         default-value
+         ok-values
+         setter-function-called-cb
+         option-widget-changed-cb)
   (define (multichoice-legal val p-vals)
     (cond ((null? p-vals) #f)
           ((eq? val (vector-ref (car p-vals) 0)) #t)
@@ -281,7 +583,10 @@
      (lambda () value)
      (lambda (x)
        (if (multichoice-legal x ok-values)
-           (set! value x)
+           (begin
+	     (set! value x)
+	     (if (procedure? setter-function-called-cb)
+		 (setter-function-called-cb x)))
            (gnc:error "Illegal Multichoice option set")))
      (lambda () default-value)
      (gnc:restore-form-generator value->string)
@@ -290,7 +595,16 @@
            (list #t x)
            (list #f "multichoice-option: illegal choice")))
      ok-values
-     (lambda () (multichoice-strings ok-values)))))
+     (vector (lambda () (length ok-values))
+       (lambda (x) (vector-ref (list-ref ok-values x) 0))
+       (lambda (x) (vector-ref (list-ref ok-values x) 1))
+       (lambda (x) (vector-ref (list-ref ok-values x) 2))
+       (lambda (x)
+	 (gnc:multichoice-list-lookup ok-values x)))
+     (lambda () (multichoice-strings ok-values)) 
+     (and option-widget-changed-cb
+	  (lambda (x) (option-widget-changed-cb x))))))
+
 
 ;; list options use the option-data in the same way as multichoice
 ;; options. List options allow the user to select more than one option.
@@ -337,8 +651,13 @@
        (if (list-legal x)
            (list #t x)
            (list #f "list-option: illegal value")))
-     ok-values
-     (lambda () (list-strings ok-values)))))
+     ok-values     
+     (vector (lambda () (length ok-values))
+       (lambda (x) (vector-ref (list-ref ok-values x) 0))
+       (lambda (x) (vector-ref (list-ref ok-values x) 1))
+       (lambda (x) (vector-ref (ref ok-values x) 2))
+       (lambda (x) (gnc:multichoice-list-lookup ok-values x)))
+     (lambda () (list-strings ok-values)) #f)))
 
 ;; number range options use the option-data as a list whose
 ;; elements are: (lower-bound upper-bound num-decimals step-size)
@@ -367,14 +686,15 @@
               (list #t x))
              (else (list #f "number-range-option: out of range"))))
      (list lower-bound upper-bound num-decimals step-size)
-     #f)))
+     #f #f #f)))
 
 (define (gnc:make-internal-option
          section
          name
          default-value)
   (let* ((value default-value)
-         (value->string (lambda () (gnc:value->string value))))
+         (value->string (lambda ()
+                          (string-append "'" (gnc:value->string value)))))
     (gnc:make-option
      section name "" 'internal #f
      (lambda () value)
@@ -382,7 +702,29 @@
      (lambda () default-value)
      (gnc:restore-form-generator value->string)
      (lambda (x) (list #t x))
-     #f #f)))
+     #f #f #f #f)))
+
+
+(define (gnc:make-query-option
+         section
+         name
+         default-value)
+  (let* ((value (if (list? default-value)
+                    default-value
+                    (gnc:query->scm default-value)))
+         (value->string (lambda ()
+                          (string-append "'" (gnc:value->string value)))))
+    (gnc:make-option
+     section name "" 'query #f
+     (lambda () value)
+     (lambda (x) (set! value (if (list? x) x (gnc:query->scm x))))
+     (lambda () (if (list? default-value)
+                    default-value
+                    (gnc:query->scm default-value)))
+     (gnc:restore-form-generator value->string)
+     (lambda (x) (list #t x))
+     #f #f #f #f)))
+
 
 ;; Color options store rgba values in a list.
 ;; The option-data is a list, whose first element
@@ -428,29 +770,35 @@
      (gnc:restore-form-generator value->string)
      validate-color
      (list range use-alpha)
-     #f)))
-
-(define (gnc:color->html color range)
+     #f #f #f)))
 
+(define (gnc:color->hex-string color range)
   (define (html-value value)
     (inexact->exact
      (min 255.0
           (truncate (* (/ 255.0 range) value)))))
-
   (let ((red (car color))
         (green (cadr color))
         (blue (caddr color)))
     (string-append
-     "#"
      (number->string (html-value red) 16)
      (number->string (html-value green) 16)
      (number->string (html-value blue) 16))))
 
+(define (gnc:color->html color range)
+    (string-append "#"
+		   (gnc:color->hex-string color range)))
+
 (define (gnc:color-option->html color-option)
   (let ((color (gnc:option-value color-option))
         (range (car (gnc:option-data color-option))))
     (gnc:color->html color range)))
 
+(define (gnc:color-option->hex-string color-option)
+  (let ((color (gnc:option-value color-option))
+        (range (car (gnc:option-data color-option))))
+    (gnc:color->hex-string color range)))
+
 
 ;; Create a new options database
 (define (gnc:new-options)
@@ -494,7 +842,7 @@
        new-option
        (lambda () (option-changed section name)))))
 
-  ; Call (thunk option) for each option in the database
+;   Call (thunk option) for each option in the database
   (define (options-for-each thunk)
     (define (section-for-each section-hash thunk)
       (hash-for-each
@@ -527,8 +875,8 @@
              (name (gnc:option-name option)))
         (string-append
          "(let ((option (gnc:lookup-option " options-string "\n"
-         "                                 \"" section "\"\n"
-         "                                 \"" name "\")))\n"
+         "                                 " (gnc:value->string section) "\n"
+         "                                 " (gnc:value->string name) ")))\n"
          "  (" restore-code " option))\n\n")))
 
     (define (generate-forms port)
@@ -540,6 +888,10 @@
        (lambda (option)
          (let ((value (gnc:option-value option))
                (default-value (gnc:option-default-value option)))
+;           (if (and (pair? default-value)); (eqv? 'commodity-scm (car value)))
+;               (begin
+;                 (write value) (newline)
+;                 (write default-value) (newline)))
            (if
             (not (equal? value default-value))
             (let* ((generator (gnc:option-generate-restore-form option))
@@ -578,11 +930,24 @@
                           (callback))))))))
 
     (if options-changed
-        (hash-for-each run-callback callback-hash))
+        (let ((cblist '()))
+          (hash-for-each 
+           (lambda (k v) (set! cblist (cons (cons k v) cblist)))
+           callback-hash)
+          (set! cblist (sort cblist 
+                             (lambda (a b) 
+                               (< (car a) (car b)))))
+          (for-each 
+           (lambda (elt) (run-callback (car elt) (cdr elt))) 
+           cblist)))
     (clear-changes))
-
+  
   (define default-section #f)
 
+  (define (touch)
+    (set! options-changed #t)
+    (run-callbacks))
+  
   (define (set-default-section section-name)
     (set! default-section section-name))
 
@@ -598,6 +963,7 @@
       ((for-each) options-for-each)
       ((for-each-general) options-for-each-general)
       ((generate-restore-forms) generate-restore-forms)
+      ((touch) touch)
       ((clear-changes) clear-changes)
       ((run-callbacks) run-callbacks)
       ((set-default-section) set-default-section)
@@ -634,6 +1000,9 @@
 (define (gnc:options-clear-changes options)
   ((options 'clear-changes)))
 
+(define (gnc:options-touch options)
+  ((options 'touch)))
+
 (define (gnc:options-run-callbacks options)
   ((options 'run-callbacks)))
 
@@ -643,37 +1012,34 @@
 (define (gnc:options-get-default-section options)
   ((options 'get-default-section)))
 
+;; Copies all values from src-options to dest-options, that is, it
+;; copies the values of all options from src which exist in dest to
+;; there.
+(define (gnc:options-copy-values src-options dest-options)
+  (if 
+   dest-options
+   (gnc:options-for-each 
+    (lambda (src-option) 
+      (let ((dest-option (gnc:lookup-option dest-options 
+					    (gnc:option-section src-option)
+					    (gnc:option-name src-option))))
+	(if dest-option
+	    (gnc:option-set-value dest-option 
+				  (gnc:option-value src-option)))))
+    src-options)))
+
 (define (gnc:send-options db_handle options)
   (gnc:options-for-each
    (lambda (option)
      (gnc:option-db-register-option db_handle option))
    options))
 
-(define (gnc:save-options options options-string file header)
+(define (gnc:save-options options options-string file header truncate?)
   (let ((code (gnc:generate-restore-forms options options-string))
-        (port (open file (logior O_WRONLY O_CREAT O_TRUNC))))
+        (port (if truncate? 
+                  (open file (logior O_WRONLY O_CREAT O_TRUNC))
+                  (open file (logior O_WRONLY O_CREAT O_APPEND)))))
     (if port (begin
                (display header port)
                (display code port)
                (close port)))))
-
-(define (gnc:options-register-translatable-strings options)
-  (define (external-name? name)
-    (cond ((not (string? name)) #f)
-          ((< (string-length name) 2) #t)
-          ((not (eq? (string-ref name 0) #\_)) #t)
-          ((not (eq? (string-ref name 1) #\_)) #t)
-          (else #f)))
-
-  (gnc:options-for-each-general
-   (lambda (section hash)
-     (if (external-name? section)
-         (gnc:register-translatable-strings section)))
-   (lambda (option)
-     (if (not (eq? (gnc:option-type option) 'internal))
-         (gnc:register-translatable-strings (gnc:option-name option)))
-     (gnc:register-translatable-strings (gnc:option-documentation option))
-     (let ((getter (gnc:option-strings-getter option)))
-       (if getter
-           (apply gnc:register-translatable-strings (getter)))))
-   options))
