@@ -3,7 +3,7 @@
 ;;;  string munging and other utility routines 
 ;;;
 ;;;  Bill Gribble <grib@billgribble.com> 20 Feb 2000 
-;;;  $Id: qif-utils.scm,v 1.6 2000/05/30 06:03:10 peticolas Exp $
+;;;  $Id: qif-utils.scm,v 1.7 2001/04/17 21:37:38 peticolas Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (gnc:support "qif-import/qif-utils.scm")
@@ -60,19 +60,6 @@
         (newstr (make-string 1 new)))
     (regexp-substitute/global #f rexpstr str 'pre newstr 'post)))
 
-(define (string-split-on str char)
-  (let ((parts '())
-        (first-char #f))
-    (let loop ((last-char (string-length str)))
-      (set! first-char (string-rindex str char 0 last-char))
-      (if first-char 
-          (begin 
-            (set! parts (cons (substring str (+ 1 first-char) last-char) 
-                              parts))
-            (loop first-char))
-          (set! parts (cons (substring str 0 last-char) parts))))    
-    parts))
-
 (define (string-to-canonical-symbol str)
   (string->symbol 
    (string-downcase
